use strict;
use Test::More tests => 10;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (JoinObjectPath ExistsObjectByPath LoadRootObject LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::WebInterface::API::FakeRequest;

## no critic qw( ProhibitMagicNumbers )

my $ROOTPATH = '/Test';

sub Main {
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;
    my $Shop = LoadObjectByPath('/Test/TestShop');
    $Shop->set({'DomainName' => 'www.myshop.lan'});

    my $Product = LoadObjectByPath('/Test/TestShop/Products/ht_0801099030');
    $Product->set({
        'Name' => 'My ht product!',
        'URI' => 'my-ht-product',
    }, $Shop->get('LanguageID'));

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

    # product canonical
    my $ProductCanonical = 'http://www.myshop.lan/my-ht-product';
    my $Response = $FakeRequest->get($ProductCanonical);
    cmp_ok( $Response->code, '==', 200, 'Product page via shorturl' );
    my $Content = $Response->decoded_content;
    ok($Content =~ /<link\s+rel=\"canonical\"\s+href=\"\Q$ProductCanonical\E\"\s+\/>/xs, 'Canonical tag ok');

    # product rating page
    my $ProductDetailUrl = $Shop->get('WebUrl').'/'.$Shop->get('LocaleID').'/'.'?ObjectPath='.$Product->get('Path').'&ViewAction=ViewProductRating';
    $Response = $FakeRequest->get($ProductDetailUrl);
    cmp_ok( $Response->code, '==', 200, 'Product rating page' );
    $Content = $Response->decoded_content;
    ok($Content =~ /<link\s+rel=\"canonical\"\s+href=\"\Q$ProductDetailUrl\E\"\s+\/>/xs, 'Canonical tag ok');


    # variation product canonical
    my $SuperProduct = LoadObjectByPath('/Test/TestShop/Products/Shirt');
    $SuperProduct->set({
        'Name' => 'My shirt product!',
        'URI' => 'my-shirt-product',
    }, $Shop->get('LanguageID'));
    my $SubProduct = LoadObjectByPath(JoinObjectPath(['Test', 'TestShop', 'Products', 'Shirt', 'SubProducts', 'Shirt blue']));

    $Response = $FakeRequest->get($SubProduct->get('WebUrl'));
    cmp_ok( $Response->code, '==', 200, 'Sub product page via weburl' );
    $Content = $Response->decoded_content;
    my $SuperProductCanonical = 'http://www.myshop.lan/my-shirt-product';
    ok($Content =~ /<link\s+rel=\"canonical\"\s+href=\"\Q$SuperProductCanonical\E\"\s+\/>/xs, 'Canonical tag points to super product');

    # variation product detail image
    $ProductDetailUrl = $Shop->get('WebUrl').'/'.$Shop->get('LocaleID').'/?ObjectPath='.$SubProduct->get('Path').'&ViewAction=ViewProductRating';
    $Response = $FakeRequest->get($ProductDetailUrl);
    cmp_ok( $Response->code, '==', 200, 'Product rating page' );
    $Content = $Response->decoded_content;
    my $SuperProductImageCanonical = $Shop->get('WebUrl').'/'.$Shop->get('LocaleID').'/?ObjectPath='.$SuperProduct->get('Path').'&ViewAction=ViewProductRating';
    ok($Content =~ /<link\s+rel=\"canonical\"\s+href=\"\Q$SuperProductImageCanonical\E\"\s+\/>/xs, 'Canonical tag points to super product rating page');

}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Debug=>1, Sub=> sub {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    Main();

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
});
