use base qw(Test::Class);
use Test::More;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError );
use DE_EPAGES::SDA::API::File::Record::DataRecordBuilder;
use DE_EPAGES::SDA::API::Object::ShippingLabelSDA
  qw ( STANDARD_SHIPPING );

my $customerIdCode1        = 'CLI000000009';
my $blanks2                = '                              ';
my $shipmentType3          = 'N';
my $activateCollection4    = 'N';
my $printConsignementNote5 = 'S';
my $shipRefNumber          = '123-ship-ref-number';

my $shipDate      = '20110501';
my $shipDateParam = DateTime->new(
    year  => 2011,
    month => 5,
    day   => 1
);
my $collectionMP     = 'P';
my $parcelsNumber    = 1;
my $productRefCode   = 'PR-5678';
my $recipientRefCode = 'CL-1234';

use constant CODE              => '12345678901234567890';
use constant CODE_BLANK        => '                    ';
use constant COMPANY           => 'My Company Ltd.';
use constant COMPANY_BLANK     => '                         ';
use constant CONTACT           => 'My Contact';
use constant CONTACT_BLANK     => '          ';
use constant STREET_TYPE_BLANK => '     ';
use constant STREET            => 'Via Margutta';
use constant STREET_BLANK      => '                  ';
use constant NUMBER            => '123';
use constant NUMBER_BLANK      => '  ';
use constant PHONE             => '555-555555';
use constant PHONE_BLANK       => '     ';
use constant POSTCODE          => '63123';
use constant POSTCODE_BLANK    => '    ';
use constant TOWN              => 'Domodossola';
use constant TOWN_BLANK        => '                   ';
use constant PROVINCE          => 'Do';
use constant PARCELS_NUMBER    => '001';
use constant WEIGHT            => '       ';
use constant WIDTH             => '     ';
use constant HEIGHT            => '     ';
use constant DEPTH             => '     ';
use constant PRD_BLANKS        => '             ';
use constant REC_BLANKS        => '             ';
use constant CURRENCY          => 'EU';
use constant CASH_AMOUNT       => '000000.00';
use constant INSURED_AMOUNT    => '0' . CASH_AMOUNT;
use constant PAYMENT_TYPE      => '   ';
use constant NOTES             => '                              ';
use constant PARCEL_IDENTIF    => '                         ';
use constant BRANCH            => 'RM';
use constant CONTENTS          => 'contents blah blah...         ';
use constant SHIPMENT_BLANK    => '      ';
use constant CONTENT_TYPE      => 'P';
use constant SENDER_COUNTRY    => 'IT';
use constant RECIPIENT_COUNTRY => 'IT';
use constant SHIPMENT_VALUE    => INSURED_AMOUNT;
use constant SHIPMENT_CLASS    => '          ';
use constant EMAIL             => 'email@example.com';
use constant MOBILE_NUMBER     => '07341234567890';
my %senderAddress = (
    'code'     => CODE,
    'person_or_company_name'  => COMPANY,
    'contact'  => CONTACT,
    'street'   => STREET . ' ' . NUMBER,
    'phone'    => PHONE,
    'postcode' => POSTCODE,
    'town'     => TOWN,
    'province' => PROVINCE,
);

my %receiverAddress = (
    'code'     => CODE,
    'person_or_company_name'  => COMPANY,
    'contact'  => CONTACT,
    'street'   => STREET . ' ' . NUMBER,
    'phone'    => PHONE,
    'postcode' => POSTCODE,
    'town'     => TOWN,
    'province' => PROVINCE,
);

my %params = (
    'customerIdCode'        => $customerIdCode1,
    'shipmentRefNumber'     => $shipRefNumber,
    'shipmentDate'          => $shipDateParam,
    'senderAddress'         => \%senderAddress,
    'receiverAddress'       => \%receiverAddress,
    'parcelsNumber'         => $parcelsNumber,
    'serviceCode'           => STANDARD_SHIPPING,
    'productRefCode'        => $productRefCode,
    'recipientRefCode'      => $recipientRefCode,
    'branch'                => BRANCH,
    'contents'              => 'contents blah blah...',
    'senderCountry'         => SENDER_COUNTRY,
    'recipientCountry'      => RECIPIENT_COUNTRY,
    'recipientEmail'        => EMAIL,
    'recipientMobileNumber' => MOBILE_NUMBER,
);

sub before : Test(setup) {
    my $self = shift;
    $self->{'builder'} =
      DE_EPAGES::SDA::API::File::Record::DataRecordBuilder->new(%params);
}

sub checkConstructor : Test(no_plan) {
    my $self    = shift;
    my $builder = $self->{'builder'};
    isa_ok( $builder, 'DE_EPAGES::SDA::API::File::Record::DataRecordBuilder' );
}

sub validateCustomerIdCode : Test(no_plan) {
    eval {
        DE_EPAGES::SDA::API::File::Record::DataRecordBuilder->new();
        fail('must throw an exception');
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        is( $Error->code, 'UNDEF_CIC', 'customer id code validation' )
          or diag( $Error->longMessage );
    }
}

sub checkRecord : Test(no_plan) {
    my $self    = shift;
    my $builder = $self->{'builder'};
    is(
        $builder->build(),
        $customerIdCode1
          . $blanks2
          . $shipmentType3
          . $activateCollection4
          . $printConsignementNote5
          . $shipRefNumber
          . SHIPMENT_BLANK
          . $shipDate
          . $collectionMP
          . CODE
          . COMPANY
          . COMPANY_BLANK
          . CONTACT
          . CONTACT_BLANK
          . STREET_TYPE_BLANK
          . STREET
          . STREET_BLANK
          . NUMBER
          . NUMBER_BLANK
          . PHONE
          . PHONE_BLANK
          . POSTCODE
          . POSTCODE_BLANK
          . TOWN
          . TOWN_BLANK
          . PROVINCE
          . CODE_BLANK
          . COMPANY
          . COMPANY_BLANK
          . CONTACT
          . CONTACT_BLANK
          . STREET_TYPE_BLANK
          . STREET
          . STREET_BLANK
          . NUMBER
          . NUMBER_BLANK
          . PHONE
          . PHONE_BLANK
          . POSTCODE
          . POSTCODE_BLANK
          . TOWN
          . TOWN_BLANK
          . PROVINCE
          . PARCELS_NUMBER
          . WEIGHT
          . WIDTH
          . HEIGHT
          . DEPTH
          . STANDARD_SHIPPING
          . $productRefCode
          . PRD_BLANKS
          . $recipientRefCode
          . REC_BLANKS
          . CURRENCY
          . CASH_AMOUNT
          . PAYMENT_TYPE
          . NOTES
          . INSURED_AMOUNT
          . PARCEL_IDENTIF
          . BRANCH
          . CONTENTS
          . CONTENT_TYPE
          . SENDER_COUNTRY . ' '
          . RECIPIENT_COUNTRY . ' '
          . SHIPMENT_VALUE . 'PCIM' . '     ' . ' ' . '  '
          . '                    ' . '   '
          . EMAIL
          . '                                 '
          . MOBILE_NUMBER,
        'data record correctly built'
    );
}

Test::Class->runtests;
