use base qw(Test::Class);
use Test::More;
use Test::MockObject;
use strict;
use DE_EPAGES::SDA::API::File::SDAFileBuilder;
use DE_EPAGES::SDA::API::FTP::SDAFTP;
use DE_EPAGES::SDA::API::Object::ShippingLabelSDA qw (STANDARD_SHIPPING);

my $orderNumber            = 'ord-1002';
my $customerIdCode1        = 'CLI000000009';
my $blanks2                = '                              ';
my $shipmentType3          = 'N';
my $activateCollection4    = 'N';
my $printConsignementNote5 = 'S';
my $shipRefNumber          = '123-ship-ref-number';

my $shipDate      = '20110501';
my $shipDateParam = DateTime->new(
    year  => 2011,
    month => 5,
    day   => 1
);
my $collectionMP     = 'P';
my $parcelsNumber    = 1;
my $weight           = 1500;
my $productRefCode   = 'PR-5678';
my $recipientRefCode = 'CL-1234';

use constant CODE              => '12345678901234567890';
use constant CODE_BLANK        => '                    ';
use constant COMPANY           => 'My Company Ltd.';
use constant COMPANY_BLANK     => '                         ';
use constant CONTACT           => 'My Contact';
use constant CONTACT_BLANK     => '          ';
use constant STREET            => 'Via Margutta';
use constant STREET_BLANK      => '                      ';
use constant NUMBER            => '123';
use constant NUMBER_BLANK      => '  ';
use constant PHONE             => '555-555555';
use constant PHONE_BLANK       => '     ';
use constant POSTCODE          => '63123';
use constant POSTCODE_BLANK    => '    ';
use constant TOWN              => 'Domodossola';
use constant TOWN_BLANK        => '                   ';
use constant PROVINCE          => 'Do';
use constant PARCELS_NUMBER    => '005';
use constant WEIGHT            => '0001500';
use constant WIDTH             => '00000';
use constant HEIGHT            => '00000';
use constant DEPTH             => '00000';
use constant PRD_BLANKS        => '             ';
use constant REC_BLANKS        => '             ';
use constant CURRENCY          => 'EU';
use constant CASH_AMOUNT       => '000000.00';
use constant INSURED_AMOUNT    => '0' . CASH_AMOUNT;
use constant PAYMENT_TYPE      => '   ';
use constant NOTES             => '                              ';
use constant PARCEL_IDENTIF    => '123-ship-ref-number-1    ';
use constant BRANCH            => 'RM';
use constant CONTENTS          => 'contents blah blah...         ';
use constant SHIPMENT_BLANK    => '      ';
use constant CONTENT_TYPE      => 'P';
use constant SENDER_COUNTRY    => 'IT';
use constant RECIPIENT_COUNTRY => 'IT';
use constant SHIPMENT_VALUE    => INSURED_AMOUNT;
use constant CUSTOMENR_TYPE    => 'PCIM';
use constant SHIPMENT_CLASS    => '          ';
use constant EMAIL             => 'email@example.com';
use constant MOBILE_PREFIX     => '0734';
use constant MOBILE_NUMBER     => '1234567890';

my %sender_address = (
    'code'     => CODE,
    'person_or_company_name'  => COMPANY,
    'contact'  => CONTACT,
    'street'   => STREET . ' ' . NUMBER,
    'phone'    => PHONE,
    'postcode' => POSTCODE,
    'town'     => TOWN,
    'province' => PROVINCE,
);

my %receiver_address = (
    'code'     => CODE,
    'person_or_company_name'  => COMPANY,
    'contact'  => CONTACT,
    'street'   => STREET . ' ' . NUMBER,
    'phone'    => PHONE,
    'postcode' => POSTCODE,
    'town'     => TOWN,
    'province' => PROVINCE,
);

my %params = (
    'date'                  => DateTime->now(),
    'orderNumber'           => $orderNumber,
    'customerIdCode'        => $customerIdCode1,
    'shipmentRefNumber'     => $shipRefNumber,
    'shipmentDate'          => $shipDateParam,
    'senderAddress'         => \%sender_address,
    'receiverAddress'       => \%receiver_address,
    'parcelsNumber'         => $parcelsNumber,
    'weight'                => $weight,
    'serviceCode'           => STANDARD_SHIPPING,
    'productRefCode'        => $productRefCode,
    'recipientRefCode'      => $recipientRefCode,
    'branch'                => BRANCH,
    'contents'              => 'contents blah blah...',
    'senderCountry'         => SENDER_COUNTRY,
    'recipientCountry'      => RECIPIENT_COUNTRY,
    'recipientEmail'        => EMAIL,
    'recipientMobilePrefix' => MOBILE_PREFIX,
    'recipientMobileNumber' => MOBILE_NUMBER,
);

sub before : Test(setup) {
    my $self = shift;
    $self->{'builder'} =
      DE_EPAGES::SDA::API::File::SDAFileBuilder->new(%params);

    $self->{'info'} = Test::MockObject->new();
    $self->{'info'}->set_always( 'host', 'localhost' );
    $self->{'info'}->set_always( 'log',  1 );

    $self->{'infoNoLogging'} = Test::MockObject->new();
    $self->{'infoNoLogging'}->set_always( 'host', 'localhost' );
    $self->{'infoNoLogging'}->set_always( 'log',  0 );

    $self->{'merchantCredentials'} = Test::MockObject->new();
    $self->{'merchantCredentials'}->set_always( 'username', 'sda1' );
    $self->{'merchantCredentials'}->set_always( 'password', 'sda1' );
}

sub putLogging : Test(no_plan) {
    my $self = shift;

    my $ftp = DE_EPAGES::SDA::API::FTP::SDAFTP->new(
        'info'                => $self->{'info'},
        'merchantCredentials' => $self->{'merchantCredentials'},
    );
    $ftp->put( $self->{'builder'} );
    ok( 1, 'put logging OK' );
}

sub putNoLogging : Test(no_plan) {
    my $self = shift;

    my $ftp = DE_EPAGES::SDA::API::FTP::SDAFTP->new(
        'info'                => $self->{'infoNoLogging'},
        'merchantCredentials' => $self->{'merchantCredentials'},
    );
    $ftp->put( $self->{'builder'} );
    ok( 1, 'put no logging OK' );
}

sub getNoLogging : Test(no_plan) {
    my $self = shift;

    my $ftp = DE_EPAGES::SDA::API::FTP::SDAFTP->new(
        'info'                => $self->{'infoNoLogging'},
        'merchantCredentials' => $self->{'merchantCredentials'},
    );
    $ftp->get();
    ok( 1, 'get no logging OK' );
}

sub getLogging : Test(no_plan) {
    my $self = shift;

    my $ftp = DE_EPAGES::SDA::API::FTP::SDAFTP->new(
        'info'                => $self->{'info'},
        'merchantCredentials' => $self->{'merchantCredentials'},
    );
    $ftp->get();
    ok( 1, 'get logging OK' );
}

Test::Class->runtests;
